"use strict";
// Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
// SPDX-License-Identifier: Apache-2.0
Object.defineProperty(exports, "__esModule", { value: true });
exports.removeListener = exports.addListenerOnce = exports.addListener = void 0;
const googleCommon_1 = require("./googleCommon");
const maps_1 = require("./maps");
const markers_1 = require("./markers");
const infoWindow_1 = require("./infoWindow");
const places_1 = require("./places");
const directions_1 = require("./directions");
function addListener(instance, eventName, handler) {
    if (instance instanceof maps_1.MigrationMap ||
        instance instanceof markers_1.MigrationMarker ||
        instance instanceof infoWindow_1.MigrationInfoWindow ||
        instance instanceof directions_1.MigrationDirectionsRenderer ||
        instance instanceof places_1.MigrationAutocomplete ||
        instance instanceof places_1.MigrationSearchBox) {
        return instance.addListener(eventName, handler);
    }
}
exports.addListener = addListener;
function addListenerOnce(instance, eventName, handler) {
    if (instance instanceof maps_1.MigrationMap ||
        instance instanceof markers_1.MigrationMarker ||
        instance instanceof infoWindow_1.MigrationInfoWindow ||
        instance instanceof directions_1.MigrationDirectionsRenderer ||
        instance instanceof places_1.MigrationAutocomplete ||
        instance instanceof places_1.MigrationSearchBox) {
        return instance.addListener(eventName, handler, "once");
    }
}
exports.addListenerOnce = addListenerOnce;
function removeListener(listener) {
    if (listener.instance instanceof maps_1.MigrationMap) {
        const map = listener.instance._getMap();
        if (googleCommon_1.GoogleMapMouseEvent.includes(listener.eventName)) {
            map.off(googleCommon_1.GoogleToMaplibreEvent[listener.eventName], listener.handler);
        }
        else if (googleCommon_1.GoogleMapEvent.includes(listener.eventName)) {
            map.off(googleCommon_1.GoogleToMaplibreEvent[listener.eventName], listener.handler);
        }
    }
    else if (listener.instance instanceof markers_1.MigrationMarker) {
        const marker = listener.instance._getMarker();
        if (googleCommon_1.GoogleMarkerMouseDOMEvent.includes(listener.eventName)) {
            marker.getElement().removeEventListener(googleCommon_1.GoogleToMaplibreEvent[listener.eventName], listener.handler);
        }
        else if (googleCommon_1.GoogleMarkerMouseEvent.includes(listener.eventName)) {
            marker.off(googleCommon_1.GoogleToMaplibreEvent[listener.eventName], listener.handler);
        }
    }
    else if (listener.instance instanceof infoWindow_1.MigrationInfoWindow) {
        const popup = listener.instance._getPopup();
        if (googleCommon_1.GoogleInfoWindowEvent.includes(listener.eventName)) {
            if (listener.eventName === googleCommon_1.MigrationEvent.close) {
                popup.off(googleCommon_1.GoogleToMaplibreEvent[listener.eventName], listener.handler);
            }
            else if (listener.eventName === googleCommon_1.MigrationEvent.closeclick) {
                const closeButton = popup.getElement().querySelector("button.maplibregl-popup-close-button");
                closeButton.removeEventListener(googleCommon_1.GoogleToMaplibreEvent[listener.eventName], listener.handler);
            }
        }
    }
    else if (listener.instance instanceof directions_1.MigrationDirectionsRenderer) {
        listener.instance[`_set${listener.listenerType}DirectionsChangedListeners`](listener.instance[`_get${listener.listenerType}DirectionsChangedListeners`]().filter((obj) => obj !== listener));
    }
    else if (listener.instance instanceof places_1.MigrationAutocomplete) {
        const geocoder = listener.instance._getMaplibreGeocoder();
        if (listener.eventName == "place_changed") {
            geocoder.getPlacesGeocoder().off("results", listener.resultsHandler);
            geocoder.getPlacesGeocoder().off("result", listener.resultHandler);
        }
    }
    else if (listener.instance instanceof places_1.MigrationSearchBox) {
        const geocoder = listener.instance._getMaplibreGeocoder();
        if (listener.eventName == "places_changed") {
            geocoder.getPlacesGeocoder().off("results", listener.resultsHandler);
            geocoder.getPlacesGeocoder().off("result", listener.resultHandler);
        }
    }
}
exports.removeListener = removeListener;
