"use strict";
// Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
// SPDX-License-Identifier: Apache-2.0
var __classPrivateFieldSet = (this && this.__classPrivateFieldSet) || function (receiver, state, value, kind, f) {
    if (kind === "m") throw new TypeError("Private method is not writable");
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a setter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot write private member to an object whose class did not declare it");
    return (kind === "a" ? f.call(receiver, value) : f ? f.value = value : state.set(receiver, value)), value;
};
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, state, kind, f) {
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a getter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot read private member from an object whose class did not declare it");
    return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
};
var _MigrationMap_instances, _MigrationMap_map, _MigrationMap_navigationControl, _MigrationMap_setMaxZoom, _MigrationMap_setMinZoom, _MigrationMap_addNavigationControl;
Object.defineProperty(exports, "__esModule", { value: true });
exports.MigrationMap = void 0;
const maplibre_gl_1 = require("maplibre-gl");
const googleCommon_1 = require("./googleCommon");
const version_1 = require("./version");
/*
  This migration map class is a thin wrapper replacement for google.maps.Map, which
  replaces Google's map with a MapLibre map and routes the appropriate APIs

  map = new google.maps.Map(document.getElementById("map"), {
    center: austinCoords,
    zoom: 11,
  });
*/
class MigrationMap {
    constructor(containerElement, options) {
        _MigrationMap_instances.add(this);
        _MigrationMap_map.set(this, void 0);
        _MigrationMap_navigationControl.set(this, void 0);
        const maplibreOptions = {
            container: containerElement,
            style: this._styleUrl,
        };
        if (options.center) {
            const lnglat = (0, googleCommon_1.LatLngToLngLat)(options.center);
            if (lnglat) {
                maplibreOptions.center = lnglat;
            }
            else {
                console.error("Unrecognized center option", options.center);
            }
        }
        // MapLibre offers 0-24 zoom (handles out of bounds), Google can potentially go higher based on location
        // see more: https://developers.google.com/maps/documentation/javascript/maxzoom
        if (options.zoom) {
            maplibreOptions.zoom = options.zoom;
        }
        if (options.maxZoom) {
            maplibreOptions.maxZoom = options.maxZoom;
        }
        if (options.minZoom) {
            maplibreOptions.minZoom = options.minZoom;
        }
        if (options.heading) {
            maplibreOptions.bearing = options.heading;
        }
        if (options.tilt) {
            maplibreOptions.pitch = options.tilt;
        }
        // Add our custom user agent header with our package version
        maplibreOptions.transformRequest = (url) => {
            return {
                url: url,
                headers: {
                    "X-Amz-User-Agent": `${navigator.userAgent} migration-sdk-${version_1.PACKAGE_VERSION}`,
                },
            };
        };
        __classPrivateFieldSet(this, _MigrationMap_map, new maplibre_gl_1.Map(maplibreOptions), "f");
        // Add NavigationControl if zoomControl is true or not passed in (Google by default adds zoom control to map),
        // furthermore, you can specify zoomControlOptions without passing in zoomControl as an option
        if (options.zoomControl === undefined || options.zoomControl === true) {
            __classPrivateFieldGet(this, _MigrationMap_instances, "m", _MigrationMap_addNavigationControl).call(this, options.zoomControlOptions);
        }
    }
    addListener(eventName, handler, listenerType = "on") {
        if (googleCommon_1.GoogleMapMouseEvent.includes(eventName)) {
            const wrappedHandler = (mapLibreMapMouseEvent) => {
                const googleMapMouseEvent = {
                    domEvent: mapLibreMapMouseEvent.originalEvent,
                    latLng: new googleCommon_1.MigrationLatLng(mapLibreMapMouseEvent.lngLat.lat, mapLibreMapMouseEvent.lngLat.lng),
                };
                handler(googleMapMouseEvent);
            };
            __classPrivateFieldGet(this, _MigrationMap_map, "f")[listenerType](googleCommon_1.GoogleToMaplibreEvent[eventName], wrappedHandler);
            return {
                instance: this,
                eventName: eventName,
                handler: wrappedHandler,
            };
        }
        else if (googleCommon_1.GoogleMapEvent.includes(eventName)) {
            __classPrivateFieldGet(this, _MigrationMap_map, "f")[listenerType](googleCommon_1.GoogleToMaplibreEvent[eventName], handler);
            return {
                instance: this,
                eventName: eventName,
                handler: handler,
            };
        }
    }
    getBounds() {
        const bounds = __classPrivateFieldGet(this, _MigrationMap_map, "f").getBounds();
        return new googleCommon_1.MigrationLatLngBounds({
            west: bounds.getWest(),
            south: bounds.getSouth(),
            east: bounds.getEast(),
            north: bounds.getNorth(),
        });
    }
    getCenter() {
        const center = __classPrivateFieldGet(this, _MigrationMap_map, "f").getCenter();
        return new googleCommon_1.MigrationLatLng(center === null || center === void 0 ? void 0 : center.lat, center === null || center === void 0 ? void 0 : center.lng);
    }
    getDiv() {
        return __classPrivateFieldGet(this, _MigrationMap_map, "f").getContainer();
    }
    getHeading() {
        return __classPrivateFieldGet(this, _MigrationMap_map, "f").getBearing();
    }
    getTilt() {
        return __classPrivateFieldGet(this, _MigrationMap_map, "f").getPitch();
    }
    getZoom() {
        return __classPrivateFieldGet(this, _MigrationMap_map, "f").getZoom();
    }
    moveCamera(cameraOptions) {
        const maplibreCameraOptions = {};
        if (cameraOptions.center) {
            const lnglat = (0, googleCommon_1.LatLngToLngLat)(cameraOptions.center);
            if (lnglat) {
                maplibreCameraOptions.center = lnglat;
            }
            else {
                console.error("Unrecognized center option", cameraOptions.center);
            }
        }
        if (cameraOptions.heading) {
            maplibreCameraOptions.bearing = cameraOptions.heading;
        }
        if (cameraOptions.tilt) {
            maplibreCameraOptions.pitch = cameraOptions.tilt;
        }
        if (cameraOptions.zoom) {
            maplibreCameraOptions.zoom = cameraOptions.zoom;
        }
        __classPrivateFieldGet(this, _MigrationMap_map, "f").jumpTo(maplibreCameraOptions);
    }
    panBy(x, y) {
        __classPrivateFieldGet(this, _MigrationMap_map, "f").panBy([x, y]);
    }
    panTo(latLng) {
        const lnglat = (0, googleCommon_1.LatLngToLngLat)(latLng);
        __classPrivateFieldGet(this, _MigrationMap_map, "f").panTo(lnglat);
    }
    setCenter(center) {
        const lnglat = (0, googleCommon_1.LatLngToLngLat)(center);
        __classPrivateFieldGet(this, _MigrationMap_map, "f").setCenter(lnglat);
    }
    setHeading(heading) {
        __classPrivateFieldGet(this, _MigrationMap_map, "f").setBearing(heading);
    }
    setOptions(options) {
        if (options.center) {
            const lnglat = (0, googleCommon_1.LatLngToLngLat)(options.center);
            if (lnglat) {
                __classPrivateFieldGet(this, _MigrationMap_map, "f").setCenter(lnglat);
            }
            else {
                console.error("Unrecognized center option", options.center);
            }
        }
        if (options.zoom) {
            this.setZoom(options.zoom);
        }
        if (options.maxZoom) {
            __classPrivateFieldGet(this, _MigrationMap_instances, "m", _MigrationMap_setMaxZoom).call(this, options.maxZoom);
        }
        if (options.minZoom) {
            __classPrivateFieldGet(this, _MigrationMap_instances, "m", _MigrationMap_setMinZoom).call(this, options.minZoom);
        }
        if (options.heading) {
            this.setHeading(options.heading);
        }
        if (options.tilt) {
            this.setTilt(options.tilt);
        }
        if (options.zoomControl === undefined) {
            // if zoomControl not sent in, check if navControl exists:  navControl exists means map created with zoomControl option
            // set to true or default, navControl not exists means map created without zoomControl option so don't create new navControl
            if (__classPrivateFieldGet(this, _MigrationMap_navigationControl, "f") !== undefined && options.zoomControlOptions && options.zoomControlOptions.position) {
                __classPrivateFieldGet(this, _MigrationMap_instances, "m", _MigrationMap_addNavigationControl).call(this, options.zoomControlOptions);
            }
        }
        else if (options.zoomControl === true) {
            __classPrivateFieldGet(this, _MigrationMap_instances, "m", _MigrationMap_addNavigationControl).call(this, options.zoomControlOptions);
        }
        else if (options.zoomControl === false) {
            __classPrivateFieldGet(this, _MigrationMap_map, "f").removeControl(__classPrivateFieldGet(this, _MigrationMap_navigationControl, "f"));
        }
    }
    setTilt(tilt) {
        __classPrivateFieldGet(this, _MigrationMap_map, "f").setPitch(tilt);
    }
    setZoom(zoom) {
        __classPrivateFieldGet(this, _MigrationMap_map, "f").setZoom(zoom);
    }
    fitBounds(bounds, padding) {
        // This will handle both LatLngBounds | LatLngBoundsLiteral input for us
        const latLngBounds = new googleCommon_1.MigrationLatLngBounds(bounds);
        if (padding !== undefined) {
            if (typeof padding === "number") {
                __classPrivateFieldGet(this, _MigrationMap_map, "f").fitBounds(latLngBounds._getBounds(), { padding: padding });
            }
            else if (typeof padding === "object") {
                __classPrivateFieldGet(this, _MigrationMap_map, "f").fitBounds(latLngBounds._getBounds(), {
                    padding: {
                        top: padding.top && typeof padding.top === "number" ? padding.top : 0,
                        bottom: padding.bottom && typeof padding.bottom === "number" ? padding.bottom : 0,
                        left: padding.left && typeof padding.left === "number" ? padding.left : 0,
                        right: padding.right && typeof padding.right === "number" ? padding.right : 0,
                    },
                });
            }
            else {
                // google does not error out on invalid padding parameter
                __classPrivateFieldGet(this, _MigrationMap_map, "f").fitBounds(latLngBounds._getBounds());
            }
        }
        else {
            __classPrivateFieldGet(this, _MigrationMap_map, "f").fitBounds(latLngBounds._getBounds());
        }
    }
    // Internal method for migration logic that needs to access the underlying MapLibre map
    _getMap() {
        return __classPrivateFieldGet(this, _MigrationMap_map, "f");
    }
    // Internal method for manually setting the private #map property (used for mocking the map in unit testing)
    _setMap(map) {
        __classPrivateFieldSet(this, _MigrationMap_map, map, "f");
    }
}
exports.MigrationMap = MigrationMap;
_MigrationMap_map = new WeakMap(), _MigrationMap_navigationControl = new WeakMap(), _MigrationMap_instances = new WeakSet(), _MigrationMap_setMaxZoom = function _MigrationMap_setMaxZoom(zoom) {
    __classPrivateFieldGet(this, _MigrationMap_map, "f").setMaxZoom(zoom);
}, _MigrationMap_setMinZoom = function _MigrationMap_setMinZoom(zoom) {
    __classPrivateFieldGet(this, _MigrationMap_map, "f").setMinZoom(zoom);
}, _MigrationMap_addNavigationControl = function _MigrationMap_addNavigationControl(zoomControlOptions) {
    // remove old navControl so we don't have multiple
    if (__classPrivateFieldGet(this, _MigrationMap_navigationControl, "f")) {
        __classPrivateFieldGet(this, _MigrationMap_map, "f").removeControl(__classPrivateFieldGet(this, _MigrationMap_navigationControl, "f"));
    }
    // add new navControl
    if (zoomControlOptions &&
        zoomControlOptions.position &&
        zoomControlOptions.position in googleCommon_1.GoogleToMaplibreControlPosition) {
        __classPrivateFieldSet(this, _MigrationMap_navigationControl, new maplibre_gl_1.NavigationControl(), "f");
        __classPrivateFieldGet(this, _MigrationMap_map, "f").addControl(__classPrivateFieldGet(this, _MigrationMap_navigationControl, "f"), googleCommon_1.GoogleToMaplibreControlPosition[zoomControlOptions.position]);
    }
    else {
        __classPrivateFieldSet(this, _MigrationMap_navigationControl, new maplibre_gl_1.NavigationControl(), "f");
        __classPrivateFieldGet(this, _MigrationMap_map, "f").addControl(__classPrivateFieldGet(this, _MigrationMap_navigationControl, "f"), "bottom-right");
    }
};
