// Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
// SPDX-License-Identifier: Apache-2.0
var __classPrivateFieldSet = (this && this.__classPrivateFieldSet) || function (receiver, state, value, kind, f) {
    if (kind === "m") throw new TypeError("Private method is not writable");
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a setter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot write private member to an object whose class did not declare it");
    return (kind === "a" ? f.call(receiver, value) : f ? f.value = value : state.set(receiver, value)), value;
};
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, state, kind, f) {
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a getter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot read private member from an object whose class did not declare it");
    return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
};
var _MigrationDirectionsRenderer_directions, _MigrationDirectionsRenderer_markers, _MigrationDirectionsRenderer_map, _MigrationDirectionsRenderer_markerOptions, _MigrationDirectionsRenderer_polylineOptions, _MigrationDirectionsRenderer_preserveViewport, _MigrationDirectionsRenderer_suppressMarkers, _MigrationDirectionsRenderer_suppressPolylines, _MigrationDirectionsRenderer_onDirectionsChangedListeners, _MigrationDirectionsRenderer_onceDirectionsChangedListeners, _MigrationDirectionsRenderer_routeIds;
import { CalculateRouteCommand, CalculateRouteMatrixCommand, } from "@aws-sdk/client-location";
import { DirectionsStatus, MigrationLatLng, MigrationLatLngBounds, PlacesServiceStatus, } from "./googleCommon";
import { MigrationMarker } from "./markers";
export var UnitSystem;
(function (UnitSystem) {
    UnitSystem[UnitSystem["IMPERIAL"] = 0] = "IMPERIAL";
    UnitSystem[UnitSystem["METRIC"] = 1] = "METRIC";
})(UnitSystem || (UnitSystem = {}));
export var TravelMode;
(function (TravelMode) {
    TravelMode["DRIVING"] = "DRIVING";
    TravelMode["WALKING"] = "WALKING";
    TravelMode["BICYCLING"] = "BICYCLING";
    TravelMode["TRANSIT"] = "TRANSIT";
    TravelMode["TWO_WHEELER"] = "TWO_WHEELER";
})(TravelMode || (TravelMode = {}));
export var DistanceMatrixStatus;
(function (DistanceMatrixStatus) {
    DistanceMatrixStatus["INVALID_REQUEST"] = "INVALID_REQUEST";
    DistanceMatrixStatus["MAX_DIMENSIONS_EXCEEDED"] = "MAX_DIMENSIONS_EXCEEDED";
    DistanceMatrixStatus["MAX_ELEMENTS_EXCEEDED"] = "MAX_ELEMENTS_EXCEEDED";
    DistanceMatrixStatus["OK"] = "OK";
    DistanceMatrixStatus["OVER_QUERY_LIMIT"] = "OVER_QUERY_LIMIT";
    DistanceMatrixStatus["REQUEST_DENIED"] = "REQUEST_DENIED";
    DistanceMatrixStatus["UNKNOWN_ERROR"] = "UNKNOWN_ERROR";
})(DistanceMatrixStatus || (DistanceMatrixStatus = {}));
export var DistanceMatrixElementStatus;
(function (DistanceMatrixElementStatus) {
    DistanceMatrixElementStatus["OK"] = "OK";
    DistanceMatrixElementStatus["ZERO_RESULTS"] = "ZERO_RESULTS";
    DistanceMatrixElementStatus["NOT_FOUND"] = "NOT_FOUND";
})(DistanceMatrixElementStatus || (DistanceMatrixElementStatus = {}));
const ASCII_CODE_A = 65;
const KILOMETERS_TO_MILES_CONSTANT = 0.621371;
const KILOMETERS_TO_METERS_CONSTANT = 1000;
// place_id and types needed for geocoded_waypoints response property, formatted_address needed for leg start_address and end_address
const ROUTE_FIND_LOCATION_FIELDS = ["geometry", "place_id", "types", "formatted_address"];
// formatted_address needed for originAddresses and destinationAddresses
const DISTANCE_MATRIX_FIND_LOCATION_FIELDS = ["geometry", "formatted_address"];
class MigrationDirectionsService {
    route(options, callback) {
        return new Promise((resolve, reject) => {
            parseOrFindLocation(options.origin, this._placesService, ROUTE_FIND_LOCATION_FIELDS)
                .then((originResponse) => {
                const departurePosition = originResponse.position;
                parseOrFindLocation(options.destination, this._placesService, ROUTE_FIND_LOCATION_FIELDS)
                    .then((destinationResponse) => {
                    const destinationPosition = destinationResponse.position;
                    const input = {
                        CalculatorName: this._routeCalculatorName,
                        DeparturePosition: departurePosition,
                        DestinationPosition: destinationPosition,
                        IncludeLegGeometry: true,
                    };
                    if ("travelMode" in options) {
                        switch (options.travelMode) {
                            case TravelMode.DRIVING: {
                                input.TravelMode = "Car";
                                break;
                            }
                            case TravelMode.WALKING: {
                                input.TravelMode = "Walking";
                                break;
                            }
                        }
                    }
                    // only pass in avoidFerries and avoidTolls options if travel mode is Driving, Amazon Location Client will error out
                    // if CarModeOptions is passed in and travel mode is not Driving
                    if (("avoidFerries" in options || "avoidTolls" in options) &&
                        "travelMode" in options &&
                        options.travelMode == TravelMode.DRIVING) {
                        const carModeOptions = {};
                        if ("avoidFerries" in options) {
                            carModeOptions.AvoidFerries = options.avoidFerries;
                        }
                        if ("avoidTolls" in options) {
                            carModeOptions.AvoidTolls = options.avoidTolls;
                        }
                        input.CarModeOptions = carModeOptions;
                    }
                    if ("drivingOptions" in options && options.travelMode == TravelMode.DRIVING) {
                        input.DepartureTime = options.drivingOptions.departureTime;
                    }
                    if ("waypoints" in options) {
                        // Array of DirectionsWaypoint
                        parseOrFindLocations(options.waypoints.map((waypoint) => waypoint.location), this._placesService, ROUTE_FIND_LOCATION_FIELDS)
                            .then((waypointResponses) => {
                            input.WaypointPositions = waypointResponses.map((locationResponse) => locationResponse.position);
                            const command = new CalculateRouteCommand(input);
                            this._client
                                .send(command)
                                .then((response) => {
                                const googleResponse = this._convertAmazonResponseToGoogleResponse(response, options, originResponse, destinationResponse, waypointResponses);
                                // if a callback was given, invoke it before resolving the promise
                                if (callback) {
                                    callback(googleResponse, DirectionsStatus.OK);
                                }
                                resolve(googleResponse);
                            })
                                .catch((error) => {
                                console.error(error);
                                reject({
                                    status: DirectionsStatus.UNKNOWN_ERROR,
                                });
                            });
                        })
                            .catch((error) => {
                            console.error(error);
                            reject({
                                status: DirectionsStatus.UNKNOWN_ERROR,
                            });
                        });
                    }
                    else {
                        const command = new CalculateRouteCommand(input);
                        this._client
                            .send(command)
                            .then((response) => {
                            const googleResponse = this._convertAmazonResponseToGoogleResponse(response, options, originResponse, destinationResponse);
                            // if a callback was given, invoke it before resolving the promise
                            if (callback) {
                                callback(googleResponse, DirectionsStatus.OK);
                            }
                            resolve(googleResponse);
                        })
                            .catch((error) => {
                            console.error(error);
                            reject({
                                status: DirectionsStatus.UNKNOWN_ERROR,
                            });
                        });
                    }
                })
                    .catch((error) => {
                    console.error(error);
                    reject({
                        status: DirectionsStatus.UNKNOWN_ERROR,
                    });
                });
            })
                .catch((error) => {
                console.error(error);
                reject({
                    status: DirectionsStatus.UNKNOWN_ERROR,
                });
            });
        });
    }
    _convertAmazonResponseToGoogleResponse(response, options, originResponse, destinationResponse, waypointResponses) {
        const bounds = response.Summary.RouteBBox;
        const googleLegs = [];
        // using "(leg) =>" instead of "function(leg)" to allow us to access 'this'
        response.Legs.forEach((leg) => {
            const steps = [];
            leg.Steps.forEach((step) => {
                steps.push({
                    distance: {
                        // we do not support Google's behavior of using the unit system of the country of origin and so we will use
                        // Amazon Location's default unit system of kilometers if the unit system option is not specified
                        text: convertKilometersToGoogleDistanceText(step.Distance, options),
                        value: step.Distance * KILOMETERS_TO_METERS_CONSTANT, // in meters, multiply km by 1000
                    },
                    duration: {
                        text: formatSecondsAsGoogleDurationText(step.DurationSeconds),
                        value: step.DurationSeconds,
                    },
                    start_location: new MigrationLatLng(step.StartPosition[1], step.StartPosition[0]),
                    end_location: new MigrationLatLng(step.EndPosition[1], step.EndPosition[0]),
                    travel_mode: options.travelMode, // TODO: For now assume the same travelMode for the request, but steps could have different individual modes
                });
            });
            googleLegs.push({
                distance: {
                    // we do not support Google's behavior of using the unit system of the country of origin and so we will use
                    // Amazon Location's default unit system of kilometers if the unit system option is not specified
                    text: convertKilometersToGoogleDistanceText(leg.Distance, options),
                    value: leg.Distance * KILOMETERS_TO_METERS_CONSTANT, // in meters, multiply km by 1000
                },
                duration: {
                    text: formatSecondsAsGoogleDurationText(leg.DurationSeconds),
                    value: leg.DurationSeconds,
                },
                geometry: leg.Geometry,
                steps: steps,
                start_location: new MigrationLatLng(leg.StartPosition[1], leg.StartPosition[0]),
                end_location: new MigrationLatLng(leg.EndPosition[1], leg.EndPosition[0]),
                start_address: originResponse.formatted_address,
                end_address: destinationResponse.formatted_address,
            });
        });
        const googleRoute = {
            bounds: new MigrationLatLngBounds({
                lng: bounds[0],
                lat: bounds[1],
            }, {
                lng: bounds[2],
                lat: bounds[3],
            }),
            legs: googleLegs,
        };
        const googleResponse = {
            request: options,
            routes: [googleRoute],
            status: DirectionsStatus.OK,
        };
        // add geocoded waypoints if the data is available
        const geocodedWaypoints = waypointResponses != null
            ? this._constructGeocodedWaypointsFromResponses(originResponse, destinationResponse, waypointResponses)
            : this._constructGeocodedWaypointsFromResponses(originResponse, destinationResponse);
        if (geocodedWaypoints != null) {
            googleResponse["geocoded_waypoints"] = geocodedWaypoints;
        }
        return googleResponse;
    }
    _constructGeocodedWaypointsFromResponses(originResponse, destinationResponse, waypointResponses) {
        const geocodedWaypoints = [];
        // add origin geocoded waypoint
        const originGeocodedWaypoint = this._constructGeocodedWaypoint(originResponse);
        if (originGeocodedWaypoint != null) {
            geocodedWaypoints.push(originGeocodedWaypoint);
        }
        // add geocoded waypoints
        if (waypointResponses != null) {
            waypointResponses.forEach((waypointResponse) => {
                const geocodedWaypoint = this._constructGeocodedWaypoint(waypointResponse);
                if (geocodedWaypoint != null) {
                    geocodedWaypoints.push(geocodedWaypoint);
                }
            });
        }
        // add destination geocoded waypoint
        const destinationGeocodedWaypoint = this._constructGeocodedWaypoint(destinationResponse);
        if (destinationGeocodedWaypoint != null) {
            geocodedWaypoints.push(destinationGeocodedWaypoint);
        }
        // if there are no geocodedWaypoints then return null
        return geocodedWaypoints.length == 0 ? null : geocodedWaypoints;
    }
    _constructGeocodedWaypoint(locationResponse) {
        const geocodedWaypoint = {};
        if (locationResponse.place_id != null) {
            geocodedWaypoint["place_id"] = locationResponse.place_id;
        }
        if (locationResponse.types != null) {
            geocodedWaypoint["types"] = locationResponse.types;
        }
        geocodedWaypoint["geocoder_status"] = DirectionsStatus.OK;
        return "place_id" in geocodedWaypoint || "types" in geocodedWaypoint ? geocodedWaypoint : null;
    }
}
class MigrationDirectionsRenderer {
    constructor(options) {
        _MigrationDirectionsRenderer_directions.set(this, void 0);
        _MigrationDirectionsRenderer_markers.set(this, void 0);
        _MigrationDirectionsRenderer_map.set(this, void 0);
        _MigrationDirectionsRenderer_markerOptions.set(this, void 0);
        _MigrationDirectionsRenderer_polylineOptions.set(this, void 0);
        _MigrationDirectionsRenderer_preserveViewport.set(this, false);
        _MigrationDirectionsRenderer_suppressMarkers.set(this, false);
        _MigrationDirectionsRenderer_suppressPolylines.set(this, false);
        _MigrationDirectionsRenderer_onDirectionsChangedListeners.set(this, []);
        _MigrationDirectionsRenderer_onceDirectionsChangedListeners.set(this, []);
        _MigrationDirectionsRenderer_routeIds.set(this, []);
        __classPrivateFieldSet(this, _MigrationDirectionsRenderer_markers, [], "f");
        this.setOptions(options);
    }
    addListener(eventName, handler, listenerType = "on") {
        if (eventName == "directions_changed") {
            // Capitalize the first letter of the listernerType string since MapLibre's method names are
            // 'On' and 'Once', not 'on' and 'once'
            if (typeof listenerType == "string" && listenerType.length > 0) {
                const capitalizedListenerType = listenerType.charAt(0).toUpperCase() + listenerType.slice(1);
                const listener = {
                    instance: this,
                    eventName: eventName,
                    handler: handler,
                    listenerType: capitalizedListenerType,
                };
                this[`_get${capitalizedListenerType}DirectionsChangedListeners`]().push(listener);
                return listener;
            }
        }
    }
    getDirections() {
        return __classPrivateFieldGet(this, _MigrationDirectionsRenderer_directions, "f");
    }
    getMap() {
        return __classPrivateFieldGet(this, _MigrationDirectionsRenderer_map, "f");
    }
    setMap(map) {
        // If we're being removed from the map, clear the directions first
        if (!map) {
            this._clearDirections();
        }
        __classPrivateFieldSet(this, _MigrationDirectionsRenderer_map, map, "f");
    }
    setDirections(directions) {
        // TODO: Currently only support one route for directions
        if (directions.routes.length !== 1) {
            return;
        }
        __classPrivateFieldSet(this, _MigrationDirectionsRenderer_directions, directions, "f");
        if (__classPrivateFieldGet(this, _MigrationDirectionsRenderer_onDirectionsChangedListeners, "f").length != 0) {
            __classPrivateFieldGet(this, _MigrationDirectionsRenderer_onDirectionsChangedListeners, "f").forEach((listener) => {
                listener.handler();
            });
        }
        if (__classPrivateFieldGet(this, _MigrationDirectionsRenderer_onceDirectionsChangedListeners, "f").length != 0) {
            while (__classPrivateFieldGet(this, _MigrationDirectionsRenderer_onceDirectionsChangedListeners, "f").length > 0) {
                // get handler then call it as a function
                __classPrivateFieldGet(this, _MigrationDirectionsRenderer_onceDirectionsChangedListeners, "f").pop().handler();
            }
        }
        // First, remove any pre-existing drawn route and its markers
        this._clearDirections();
        const route = directions.routes[0];
        // Adjust the map to fit to the bounds for this route if preserveViewport option is not set to true
        if (__classPrivateFieldGet(this, _MigrationDirectionsRenderer_preserveViewport, "f") === false) {
            const boundsPaddingInPixels = 100;
            __classPrivateFieldGet(this, _MigrationDirectionsRenderer_map, "f").fitBounds(route.bounds, boundsPaddingInPixels);
        }
        const maplibreMap = __classPrivateFieldGet(this, _MigrationDirectionsRenderer_map, "f")._getMap();
        for (let i = 0; i < route.legs.length; i++) {
            const leg = route.legs[i];
            // leg.geometry is a new field we've added, because Google doesn't provide the polyline
            // for the leg as a whole, only for the individual steps, but our API (currently) only provides
            // a polyline for the entire leg.
            const geometry = leg.geometry;
            // TODO: Detect geometry type instead of just doing LineString
            if (__classPrivateFieldGet(this, _MigrationDirectionsRenderer_suppressPolylines, "f") === false) {
                const routeId = "route" + i;
                maplibreMap.addSource(routeId, {
                    type: "geojson",
                    data: {
                        type: "Feature",
                        properties: {},
                        geometry: {
                            type: "LineString",
                            coordinates: geometry.LineString,
                        },
                    },
                });
                // 8 weight, 0.5 opacity, "#73B9FF" color for default, 3 weight, 1 opacity, "Black" color used when one property is set
                const paintOptions = {};
                if (__classPrivateFieldGet(this, _MigrationDirectionsRenderer_polylineOptions, "f")) {
                    paintOptions["line-color"] = __classPrivateFieldGet(this, _MigrationDirectionsRenderer_polylineOptions, "f").strokeColor ? __classPrivateFieldGet(this, _MigrationDirectionsRenderer_polylineOptions, "f").strokeColor : "Black";
                    paintOptions["line-width"] = __classPrivateFieldGet(this, _MigrationDirectionsRenderer_polylineOptions, "f").strokeWeight ? __classPrivateFieldGet(this, _MigrationDirectionsRenderer_polylineOptions, "f").strokeWeight : 3;
                    paintOptions["line-opacity"] = __classPrivateFieldGet(this, _MigrationDirectionsRenderer_polylineOptions, "f").strokeOpacity ? __classPrivateFieldGet(this, _MigrationDirectionsRenderer_polylineOptions, "f").strokeOpacity : 1;
                }
                else {
                    // default line
                    paintOptions["line-color"] = "#73B9FF";
                    paintOptions["line-width"] = 8;
                    paintOptions["line-opacity"] = 0.5;
                }
                maplibreMap.addLayer({
                    id: routeId,
                    type: "line",
                    source: routeId,
                    layout: {
                        "line-join": "round",
                        "line-cap": "round",
                        visibility: __classPrivateFieldGet(this, _MigrationDirectionsRenderer_polylineOptions, "f") && __classPrivateFieldGet(this, _MigrationDirectionsRenderer_polylineOptions, "f").visible == false ? "none" : "visible",
                    },
                    paint: paintOptions,
                });
                __classPrivateFieldGet(this, _MigrationDirectionsRenderer_routeIds, "f").push(routeId);
            }
            // Add markers for the start location of the current leg
            if (__classPrivateFieldGet(this, _MigrationDirectionsRenderer_suppressMarkers, "f") === false) {
                const startLocation = leg.start_location;
                const startMarkerOptions = __classPrivateFieldGet(this, _MigrationDirectionsRenderer_markerOptions, "f") === undefined
                    ? { label: String.fromCharCode(ASCII_CODE_A + i) }
                    : structuredClone(__classPrivateFieldGet(this, _MigrationDirectionsRenderer_markerOptions, "f"));
                startMarkerOptions.position = startLocation;
                startMarkerOptions.map = __classPrivateFieldGet(this, _MigrationDirectionsRenderer_map, "f");
                const startMarker = new MigrationMarker(startMarkerOptions);
                __classPrivateFieldGet(this, _MigrationDirectionsRenderer_markers, "f").push(startMarker);
            }
            // TODO: Add default info windows once location information is passed into route result
        }
        // Add final marker for end location of enture route
        if (__classPrivateFieldGet(this, _MigrationDirectionsRenderer_suppressMarkers, "f") === false) {
            const lastLeg = route.legs[route.legs.length - 1];
            const endMarkerOptions = __classPrivateFieldGet(this, _MigrationDirectionsRenderer_markerOptions, "f") === undefined
                ? { label: String.fromCharCode(ASCII_CODE_A + route.legs.length) }
                : structuredClone(__classPrivateFieldGet(this, _MigrationDirectionsRenderer_markerOptions, "f"));
            endMarkerOptions.position = lastLeg.end_location;
            endMarkerOptions.map = __classPrivateFieldGet(this, _MigrationDirectionsRenderer_map, "f");
            const endMarker = new MigrationMarker(endMarkerOptions);
            __classPrivateFieldGet(this, _MigrationDirectionsRenderer_markers, "f").push(endMarker);
        }
    }
    setOptions(options) {
        if (options !== undefined && "map" in options) {
            this.setMap(options.map);
        }
        if (options !== undefined && "markerOptions" in options) {
            __classPrivateFieldSet(this, _MigrationDirectionsRenderer_markerOptions, options.markerOptions, "f");
        }
        if (options !== undefined && "preserveViewport" in options) {
            __classPrivateFieldSet(this, _MigrationDirectionsRenderer_preserveViewport, options.preserveViewport, "f");
        }
        if (options !== undefined && "directions" in options) {
            this.setDirections(options.directions);
        }
        if (options !== undefined && "suppressMarkers" in options) {
            __classPrivateFieldSet(this, _MigrationDirectionsRenderer_suppressMarkers, options.suppressMarkers, "f");
        }
        if (options !== undefined && "suppressPolylines" in options) {
            __classPrivateFieldSet(this, _MigrationDirectionsRenderer_suppressPolylines, options.suppressPolylines, "f");
        }
        if (options !== undefined && "polylineOptions" in options) {
            __classPrivateFieldSet(this, _MigrationDirectionsRenderer_polylineOptions, options.polylineOptions, "f");
        }
    }
    _clearDirections() {
        if (__classPrivateFieldGet(this, _MigrationDirectionsRenderer_markers, "f").length) {
            __classPrivateFieldGet(this, _MigrationDirectionsRenderer_markers, "f").forEach(function (marker) {
                marker.remove();
            });
            __classPrivateFieldSet(this, _MigrationDirectionsRenderer_markers, [], "f");
        }
        if (__classPrivateFieldGet(this, _MigrationDirectionsRenderer_routeIds, "f").length) {
            const maplibreMap = __classPrivateFieldGet(this, _MigrationDirectionsRenderer_map, "f")._getMap();
            __classPrivateFieldGet(this, _MigrationDirectionsRenderer_routeIds, "f").forEach(function (routeId) {
                maplibreMap.removeLayer(routeId);
                maplibreMap.removeSource(routeId);
            });
            __classPrivateFieldSet(this, _MigrationDirectionsRenderer_routeIds, [], "f");
        }
    }
    _getMarkers() {
        return __classPrivateFieldGet(this, _MigrationDirectionsRenderer_markers, "f");
    }
    _getMarkerOptions() {
        return __classPrivateFieldGet(this, _MigrationDirectionsRenderer_markerOptions, "f");
    }
    _getPreserveViewport() {
        return __classPrivateFieldGet(this, _MigrationDirectionsRenderer_preserveViewport, "f");
    }
    _getSuppressMarkers() {
        return __classPrivateFieldGet(this, _MigrationDirectionsRenderer_suppressMarkers, "f");
    }
    _getSuppressPolylines() {
        return __classPrivateFieldGet(this, _MigrationDirectionsRenderer_suppressPolylines, "f");
    }
    _getOnDirectionsChangedListeners() {
        return __classPrivateFieldGet(this, _MigrationDirectionsRenderer_onDirectionsChangedListeners, "f");
    }
    _getOnceDirectionsChangedListeners() {
        return __classPrivateFieldGet(this, _MigrationDirectionsRenderer_onceDirectionsChangedListeners, "f");
    }
    _setOnDirectionsChangedListeners(listeners) {
        __classPrivateFieldSet(this, _MigrationDirectionsRenderer_onDirectionsChangedListeners, listeners, "f");
    }
    _setOnceDirectionsChangedListeners(listeners) {
        __classPrivateFieldSet(this, _MigrationDirectionsRenderer_onceDirectionsChangedListeners, listeners, "f");
    }
}
_MigrationDirectionsRenderer_directions = new WeakMap(), _MigrationDirectionsRenderer_markers = new WeakMap(), _MigrationDirectionsRenderer_map = new WeakMap(), _MigrationDirectionsRenderer_markerOptions = new WeakMap(), _MigrationDirectionsRenderer_polylineOptions = new WeakMap(), _MigrationDirectionsRenderer_preserveViewport = new WeakMap(), _MigrationDirectionsRenderer_suppressMarkers = new WeakMap(), _MigrationDirectionsRenderer_suppressPolylines = new WeakMap(), _MigrationDirectionsRenderer_onDirectionsChangedListeners = new WeakMap(), _MigrationDirectionsRenderer_onceDirectionsChangedListeners = new WeakMap(), _MigrationDirectionsRenderer_routeIds = new WeakMap();
class MigrationDistanceMatrixService {
    getDistanceMatrix(request, callback) {
        return new Promise((resolve, reject) => {
            parseOrFindLocations(request.origins, this._placesService, DISTANCE_MATRIX_FIND_LOCATION_FIELDS)
                .then((originsResponse) => {
                parseOrFindLocations(request.destinations, this._placesService, DISTANCE_MATRIX_FIND_LOCATION_FIELDS)
                    .then((destinationsResponse) => {
                    const input = {
                        CalculatorName: this._routeCalculatorName,
                        DeparturePositions: originsResponse.map((originResponse) => originResponse.position),
                        DestinationPositions: destinationsResponse.map((destinationResponse) => destinationResponse.position), // required
                    };
                    if ("travelMode" in request) {
                        switch (request.travelMode) {
                            case TravelMode.DRIVING: {
                                input.TravelMode = "Car";
                                break;
                            }
                            case TravelMode.WALKING: {
                                input.TravelMode = "Walking";
                                break;
                            }
                        }
                    }
                    // only pass in avoidFerries and avoidTolls options if travel mode is Driving, Amazon Location Client will error out
                    // if CarModeOptions is passed in and travel mode is not Driving
                    if (("avoidFerries" in request || "avoidTolls" in request) &&
                        "travelMode" in request &&
                        request.travelMode == TravelMode.DRIVING) {
                        const carModeOptions = {};
                        if ("avoidFerries" in request) {
                            carModeOptions.AvoidFerries = request.avoidFerries;
                        }
                        if ("avoidTolls" in request) {
                            carModeOptions.AvoidTolls = request.avoidTolls;
                        }
                        input.CarModeOptions = carModeOptions;
                    }
                    if ("drivingOptions" in request && request.travelMode == TravelMode.DRIVING) {
                        input.DepartureTime = request.drivingOptions.departureTime;
                    }
                    const command = new CalculateRouteMatrixCommand(input);
                    this._client
                        .send(command)
                        .then((response) => {
                        const googleResponse = this._convertAmazonResponseToGoogleResponse(response, originsResponse, destinationsResponse, request);
                        // if a callback was given, invoke it before resolving the promise
                        if (callback) {
                            callback(googleResponse, DistanceMatrixStatus.OK);
                        }
                        resolve(googleResponse);
                    })
                        .catch((error) => {
                        console.error(error);
                        reject({
                            status: DistanceMatrixStatus.UNKNOWN_ERROR,
                        });
                    });
                })
                    .catch((error) => {
                    console.error(error);
                    reject({
                        status: DistanceMatrixStatus.UNKNOWN_ERROR,
                    });
                });
            })
                .catch((error) => {
                console.error(error);
                reject({
                    status: DistanceMatrixStatus.UNKNOWN_ERROR,
                });
            });
        });
    }
    _convertAmazonResponseToGoogleResponse(calculateRouteMatrixResponse, originsResponse, destinationsResponse, request) {
        const distanceMatrixResponseRows = [];
        calculateRouteMatrixResponse.RouteMatrix.forEach((row) => {
            const distanceMatrixResponseRow = {
                elements: [],
            };
            row.forEach((cell) => {
                // add element with response data to row
                distanceMatrixResponseRow.elements.push({
                    distance: {
                        text: convertKilometersToGoogleDistanceText(cell.Distance, request),
                        value: cell.Distance * KILOMETERS_TO_METERS_CONSTANT,
                    },
                    duration: {
                        text: formatSecondsAsGoogleDurationText(cell.DurationSeconds),
                        value: cell.DurationSeconds,
                    },
                    status: DistanceMatrixElementStatus.OK,
                });
            });
            distanceMatrixResponseRows.push(distanceMatrixResponseRow);
        });
        // TODO: add destinationAddresses and originAddresses to response using destinationsResponse and originsResponse
        // once PlacesService can reverse geocode (need to retrieve address name for coordinates to add to address arrays)
        const distanceMatrixResponse = {
            originAddresses: [],
            destinationAddresses: [],
            rows: distanceMatrixResponseRows,
        };
        return distanceMatrixResponse;
    }
}
function parseOrFindLocations(locationInputs, placesService, findPlaceFromQueryFields) {
    const locations = [];
    for (const locationInput of locationInputs) {
        locations.push(parseOrFindLocation(locationInput, placesService, findPlaceFromQueryFields));
    }
    return Promise.all(locations);
}
function parseOrFindLocation(locationInput, placesService, findPlaceFromQueryFields) {
    // The locationInput can be either a string to be geocoded, a Place, LatLng or LatLngLiteral
    // For query or placeId, we will need to perform a request to figure out the location.
    // Otherwise, for LatLng|LatLngLiteral we can just parse it.
    return new Promise((resolve, reject) => {
        // For a query, we use findPlaceFromQuery to retrieve the location
        if (typeof locationInput === "string" || typeof (locationInput === null || locationInput === void 0 ? void 0 : locationInput.query) === "string") {
            const query = (locationInput === null || locationInput === void 0 ? void 0 : locationInput.query) || locationInput;
            const findPlaceFromQueryRequest = {
                query: query,
                fields: findPlaceFromQueryFields,
            };
            placesService.findPlaceFromQuery(findPlaceFromQueryRequest, (results, status) => {
                if (status === PlacesServiceStatus.OK && results.length) {
                    const locationLatLng = results[0].geometry.location;
                    const position = [locationLatLng.lng(), locationLatLng.lat()];
                    resolve({
                        locationLatLng: locationLatLng,
                        position: position,
                        place_id: results[0].place_id,
                        types: results[0].types,
                        formatted_address: results[0].formatted_address,
                    });
                }
                else {
                    reject({});
                }
            });
        }
        else if (typeof (locationInput === null || locationInput === void 0 ? void 0 : locationInput.placeId) === "string") {
            // For a Place object with placeId, we use getDetails to retrieve the location
            const getDetailsRequest = {
                placeId: locationInput.placeId,
            };
            placesService.getDetails(getDetailsRequest, function (result, status) {
                if (status === PlacesServiceStatus.OK) {
                    const locationLatLng = result.geometry.location;
                    const position = [locationLatLng.lng(), locationLatLng.lat()];
                    resolve({
                        locationLatLng: locationLatLng,
                        position: position,
                        place_id: locationInput === null || locationInput === void 0 ? void 0 : locationInput.placeId,
                        types: result.types,
                        formatted_address: result.formatted_address,
                    });
                }
                else {
                    reject({});
                }
            });
        }
        else {
            // Otherwise, it's a LatLng|LatLngLiteral (explicitly or as Place.location)
            const latLngOrLiteral = (locationInput === null || locationInput === void 0 ? void 0 : locationInput.location) || locationInput;
            const latLng = new MigrationLatLng(latLngOrLiteral);
            resolve({
                locationLatLng: latLng,
                position: [latLng.lng(), latLng.lat()],
            });
        }
    });
}
function formatSecondsAsGoogleDurationText(seconds) {
    // convert seconds to days, hours, and minutes, rounding up to whole minutes
    const days = Math.floor(seconds / 86400); // 1 day = 86400 seconds
    const remainingSeconds = seconds % 86400;
    const hours = Math.floor(remainingSeconds / 3600);
    const remainingMinuteSeconds = remainingSeconds % 3600;
    const minutes = Math.ceil(remainingMinuteSeconds / 60);
    // take care of the "1 day", "1 hour", or "1 min" edge case
    const dayString = days > 0 ? `${days === 1 ? `${days} day` : `${days} days`}` : "";
    const hourString = hours > 0 ? `${hours === 1 ? `${hours} hour` : `${hours} hours`}` : "";
    const minuteString = minutes > 0 ? `${minutes === 1 ? `${minutes} min` : `${minutes} mins`}` : "";
    // return day, hour, and minute strings only if they are set
    const parts = [dayString, hourString, minuteString].filter((str) => str !== "");
    return parts.join(" ");
}
function convertKilometersToGoogleDistanceText(kilometers, options) {
    return "unitSystem" in options && options.unitSystem == UnitSystem.IMPERIAL
        ? kilometers * KILOMETERS_TO_MILES_CONSTANT + " mi"
        : kilometers + " km";
}
export { MigrationDirectionsService, MigrationDirectionsRenderer, MigrationDistanceMatrixService };
