// Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
// SPDX-License-Identifier: Apache-2.0
var __classPrivateFieldSet = (this && this.__classPrivateFieldSet) || function (receiver, state, value, kind, f) {
    if (kind === "m") throw new TypeError("Private method is not writable");
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a setter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot write private member to an object whose class did not declare it");
    return (kind === "a" ? f.call(receiver, value) : f ? f.value = value : state.set(receiver, value)), value;
};
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, state, kind, f) {
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a getter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot read private member from an object whose class did not declare it");
    return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
};
var _MigrationInfoWindow_popup, _MigrationInfoWindow_minWidth, _MigrationInfoWindow_maxWidth, _MigrationInfoWindow_ariaLabel;
import { Popup } from "maplibre-gl";
import { GoogleInfoWindowEvent, GoogleToMaplibreEvent, LatLngToLngLat, MigrationEvent, MigrationLatLng, } from "./googleCommon";
import { addListenerOnce } from "./events";
const focusQuerySelector = [
    "a[href]",
    "[tabindex]:not([tabindex='-1'])",
    "[contenteditable]:not([contenteditable='false'])",
    "button:not([disabled])",
    "input:not([disabled])",
    "select:not([disabled])",
    "textarea:not([disabled])",
].join(", ");
class MigrationInfoWindow {
    constructor(options) {
        _MigrationInfoWindow_popup.set(this, void 0);
        _MigrationInfoWindow_minWidth.set(this, void 0);
        _MigrationInfoWindow_maxWidth.set(this, void 0);
        _MigrationInfoWindow_ariaLabel.set(this, void 0);
        const maplibreOptions = {};
        // maxWidth - set MapLibre 'maxWidth' option
        if ("maxWidth" in options) {
            maplibreOptions.maxWidth = options.maxWidth + "px";
            __classPrivateFieldSet(this, _MigrationInfoWindow_maxWidth, options.maxWidth, "f");
        }
        // needed so that popup won't close when map is clicked to match Google (default is true)
        maplibreOptions.closeOnClick = false;
        __classPrivateFieldSet(this, _MigrationInfoWindow_popup, new Popup(maplibreOptions), "f");
        // content can be string, HTMLElement, or string containing HTML
        if ("content" in options) {
            this.setContent(options.content);
        }
        if ("position" in options) {
            this.setPosition(options.position);
        }
        // cannot set minWidth on popup if popup is not open (on the DOM tree), have to store in local variable
        if ("minWidth" in options) {
            __classPrivateFieldSet(this, _MigrationInfoWindow_minWidth, options.minWidth, "f");
        }
        // cannot set minWidth on popup if popup is not open (on the DOM tree), have to store in local variable
        if ("ariaLabel" in options) {
            __classPrivateFieldSet(this, _MigrationInfoWindow_ariaLabel, options.ariaLabel, "f");
        }
    }
    addListener(eventName, handler, listenerType = "on") {
        if (GoogleInfoWindowEvent.includes(eventName)) {
            // if close then use 'on' method on popup instance
            if (eventName === MigrationEvent.close) {
                __classPrivateFieldGet(this, _MigrationInfoWindow_popup, "f")[listenerType](GoogleToMaplibreEvent[eventName], handler);
                return {
                    instance: this,
                    eventName: eventName,
                    handler: handler,
                };
            }
            else if (eventName === MigrationEvent.closeclick) {
                // if closeclick then use 'addEventListener' method on button element
                const closeButton = __classPrivateFieldGet(this, _MigrationInfoWindow_popup, "f").getElement().querySelector("button.maplibregl-popup-close-button");
                const wrappedHandler = () => {
                    handler();
                    if (listenerType == "once") {
                        closeButton.removeEventListener(GoogleToMaplibreEvent[eventName], wrappedHandler);
                    }
                };
                closeButton.addEventListener(GoogleToMaplibreEvent[eventName], wrappedHandler);
                return {
                    instance: this,
                    eventName: eventName,
                    handler: wrappedHandler,
                };
            }
        }
    }
    close() {
        __classPrivateFieldGet(this, _MigrationInfoWindow_popup, "f").remove();
    }
    focus() {
        const container = __classPrivateFieldGet(this, _MigrationInfoWindow_popup, "f").getElement();
        // popup/infowindow not yet rendered
        if (container === undefined) {
            console.error("InfoWindow is not visible");
            return;
        }
        const firstFocusable = container.querySelector(focusQuerySelector);
        if (firstFocusable) {
            firstFocusable.focus();
        }
    }
    getContent() {
        return __classPrivateFieldGet(this, _MigrationInfoWindow_popup, "f")._content;
    }
    getPosition() {
        const position = __classPrivateFieldGet(this, _MigrationInfoWindow_popup, "f").getLngLat();
        return new MigrationLatLng(position === null || position === void 0 ? void 0 : position.lat, position === null || position === void 0 ? void 0 : position.lng);
    }
    // Google:
    // - both Marker and LatLng popup/infowindow -> set Marker or LatLng in InfoWindow class and then call InfoWindow.open
    // MapLibre:
    // - Marker popup/infowindow -> Marker.setPopup then Marker.togglePopup (to open Popup)
    // - LatLng popup/infowindow -> Popup.setLngLat (in options when creating popup) then Popup.addTo
    open(options, anchor) {
        // close popup if it is already open (to match Google)
        this.close();
        if (options && "shouldFocus" in options) {
            __classPrivateFieldGet(this, _MigrationInfoWindow_popup, "f").options.focusAfterOpen = options.shouldFocus;
        }
        if (anchor || options.anchor) {
            // Marker specific info window
            const marker = anchor !== undefined ? anchor._getMarker() : options.anchor._getMarker();
            marker.setPopup(__classPrivateFieldGet(this, _MigrationInfoWindow_popup, "f"));
            if (!__classPrivateFieldGet(this, _MigrationInfoWindow_popup, "f").isOpen()) {
                marker.togglePopup();
            }
            // Needed so that popup is not associated with marker when popup is closed
            // (otherwise Maplibre default behavior will allow users to click the marker and open the popup again
            // which does not match Google's behavior)
            addListenerOnce(this, "close", () => {
                marker.setPopup(null);
            });
            addListenerOnce(this, "closeclick", () => {
                marker.setPopup(null);
            });
        }
        else if (options.map) {
            // LatLng specific info window
            __classPrivateFieldGet(this, _MigrationInfoWindow_popup, "f").addTo(options.map._getMap());
        }
        // set style property to local variable once popup is opened
        if (__classPrivateFieldGet(this, _MigrationInfoWindow_minWidth, "f") !== undefined) {
            __classPrivateFieldGet(this, _MigrationInfoWindow_popup, "f").getElement().style.minWidth = __classPrivateFieldGet(this, _MigrationInfoWindow_minWidth, "f") + "px";
        }
        // use setMaxWidth MapLibre method if maxWidth is set in setOptions Google method
        if (__classPrivateFieldGet(this, _MigrationInfoWindow_maxWidth, "f") !== undefined) {
            __classPrivateFieldGet(this, _MigrationInfoWindow_popup, "f").setMaxWidth(__classPrivateFieldGet(this, _MigrationInfoWindow_maxWidth, "f") + "px");
        }
        // set ariaLabel property to local variable once popup is opened
        if (__classPrivateFieldGet(this, _MigrationInfoWindow_ariaLabel, "f") !== undefined) {
            __classPrivateFieldGet(this, _MigrationInfoWindow_popup, "f").getElement().ariaLabel = __classPrivateFieldGet(this, _MigrationInfoWindow_ariaLabel, "f");
        }
    }
    setContent(content) {
        if (typeof content === "string") {
            if (this._containsOnlyHTMLElements(content)) {
                __classPrivateFieldGet(this, _MigrationInfoWindow_popup, "f").setHTML(content);
            }
            else {
                __classPrivateFieldGet(this, _MigrationInfoWindow_popup, "f").setText(content);
            }
        }
        else if (content instanceof HTMLElement) {
            __classPrivateFieldGet(this, _MigrationInfoWindow_popup, "f").setDOMContent(content);
        }
    }
    setOptions(options) {
        // have to close and reopen infowindowto register new minWidth
        if ("minWidth" in options) {
            __classPrivateFieldSet(this, _MigrationInfoWindow_minWidth, options.minWidth, "f");
        }
        // have to close and reopen infowindow to register new maxWidth
        if ("maxWidth" in options) {
            __classPrivateFieldSet(this, _MigrationInfoWindow_maxWidth, options.maxWidth, "f");
        }
        // applies new ariaLabel immediately
        if ("ariaLabel" in options) {
            __classPrivateFieldGet(this, _MigrationInfoWindow_popup, "f").getElement().ariaLabel = options.ariaLabel;
            __classPrivateFieldSet(this, _MigrationInfoWindow_ariaLabel, options.ariaLabel, "f");
        }
        // applies new content immediately
        if ("content" in options) {
            this.setContent(options.content);
        }
        // applies new position immediately
        if ("position" in options) {
            this.setPosition(options.position);
        }
    }
    setPosition(position) {
        if (position) {
            const lnglat = LatLngToLngLat(position);
            __classPrivateFieldGet(this, _MigrationInfoWindow_popup, "f").setLngLat(lnglat);
        }
    }
    // Internal method for manually getting the private #popup property
    _getPopup() {
        return __classPrivateFieldGet(this, _MigrationInfoWindow_popup, "f");
    }
    // Internal method for manually getting the private #minWidth property
    _getMinWidth() {
        return __classPrivateFieldGet(this, _MigrationInfoWindow_minWidth, "f");
    }
    // Internal method for manually getting the private #maxWidth property
    _getMaxWidth() {
        return __classPrivateFieldGet(this, _MigrationInfoWindow_maxWidth, "f");
    }
    // Internal method for manually getting the private #ariaLabel property
    _getAriaLabel() {
        return __classPrivateFieldGet(this, _MigrationInfoWindow_ariaLabel, "f");
    }
    // Internal method for manually setting the private #popup property (used for mocking the marker in unit testing)
    _setPopup(popup) {
        __classPrivateFieldSet(this, _MigrationInfoWindow_popup, popup, "f");
    }
    // Internal method for manually setting the private #maxWidth property
    _setMaxWidth(maxWidth) {
        __classPrivateFieldSet(this, _MigrationInfoWindow_maxWidth, maxWidth, "f");
    }
    // Internal method for checking if a string contains valid HTML
    _containsOnlyHTMLElements(str) {
        // Regular expression to match complete HTML elements (opening and closing tags with content)
        const htmlElementRegex = /<([a-z][a-z0-9]*)\b[^>]*>(.*?)<\/\1>/gi;
        // Check if the string contains any complete HTML elements
        const hasHTMLElements = str.match(htmlElementRegex) !== null;
        // Check if the string contains any text outside of HTML elements
        const textOutsideElements = str.replace(htmlElementRegex, "").trim();
        return hasHTMLElements && textOutsideElements.length === 0;
    }
}
_MigrationInfoWindow_popup = new WeakMap(), _MigrationInfoWindow_minWidth = new WeakMap(), _MigrationInfoWindow_maxWidth = new WeakMap(), _MigrationInfoWindow_ariaLabel = new WeakMap();
export { MigrationInfoWindow };
