// Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
// SPDX-License-Identifier: Apache-2.0
var __classPrivateFieldSet = (this && this.__classPrivateFieldSet) || function (receiver, state, value, kind, f) {
    if (kind === "m") throw new TypeError("Private method is not writable");
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a setter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot write private member to an object whose class did not declare it");
    return (kind === "a" ? f.call(receiver, value) : f ? f.value = value : state.set(receiver, value)), value;
};
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, state, kind, f) {
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a getter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot read private member from an object whose class did not declare it");
    return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
};
var _MigrationAutocomplete_maplibreGeocoder, _MigrationAutocomplete_bounds, _MigrationAutocomplete_strictBounds, _MigrationAutocomplete_fields, _MigrationAutocomplete_place, _MigrationSearchBox_maplibreGeocoder, _MigrationSearchBox_bounds, _MigrationSearchBox_places;
import { buildAmazonLocationMaplibreGeocoder, } from "@aws/amazon-location-for-maplibre-gl-geocoder";
import { GetPlaceCommand, SearchPlaceIndexForSuggestionsCommand, SearchPlaceIndexForTextCommand, } from "@aws-sdk/client-location";
import { LatLngToLngLat, MigrationLatLng, MigrationLatLngBounds, PlacesServiceStatus, } from "./googleCommon";
// This helper is for converting an Amazon Place object to the legacy Google Places object format
const convertAmazonPlaceToGoogle = (placeObject, fields, includeDetailFields) => {
    const place = placeObject.Place;
    const googlePlace = {};
    // For findPlaceFromQuery, the fields are required.
    // But for getDetails, they are optional, and if they aren't specified
    // then it is the same as requesting all fields.
    let includeAllFields = false;
    if (!fields || fields.includes("ALL")) {
        includeAllFields = true;
    }
    if (includeAllFields || fields.includes("formatted_address")) {
        googlePlace["formatted_address"] = place.Label;
    }
    if (includeAllFields || fields.includes("geometry") || fields.includes("geometry.location")) {
        const point = place.Geometry.Point;
        googlePlace["geometry"] = {
            location: new MigrationLatLng(point[1], point[0]),
        };
    }
    if (includeAllFields || fields.includes("name")) {
        googlePlace["name"] = place.Label.split(",")[0];
    }
    if (includeAllFields || fields.includes("place_id")) {
        googlePlace["place_id"] = placeObject.PlaceId;
    }
    if (includeAllFields || fields.includes("reference")) {
        googlePlace["reference"] = placeObject.PlaceId;
    }
    // Needed for MigrationDirectionsService.route method's response field "geocoded_waypoints"
    // which needs DirectionsGeocodedWaypoint objects that have property "type" which is the
    // equivalent of Amazon Location's "Categories" property
    if ((includeAllFields || fields.includes("types")) && place.Categories != null) {
        googlePlace["types"] = place.Categories;
    }
    // Handle additional fields for getDetails request
    if (includeDetailFields) {
        // Our time zone offset is given in seconds, but Google's uses minutes
        // Google's utc_offset field is deprecated in favor of utc_offset_minutes,
        // but they still support it so we support both
        let timeZoneOffsetInMinutes;
        if (place.TimeZone) {
            timeZoneOffsetInMinutes = place.TimeZone.Offset / 60;
        }
        if (includeAllFields || fields.includes("utc_offset")) {
            googlePlace["utc_offset"] = timeZoneOffsetInMinutes;
        }
        if (includeAllFields || fields.includes("utc_offset_minutes")) {
            googlePlace["utc_offset_minutes"] = timeZoneOffsetInMinutes;
        }
        // vicinity is in the format of "AddressNumber Street, Municipality",
        // but street number or name might not be there depending on what was
        // searched for (e.g. just a city name)
        if (includeAllFields || fields.includes("vicinity")) {
            let vicinity = place.Municipality;
            if (place.Street) {
                vicinity = `${place.Street}, ${vicinity}`;
            }
            if (place.AddressNumber) {
                vicinity = `${place.AddressNumber} ${vicinity}`;
            }
            googlePlace["vicinity"] = vicinity;
        }
    }
    return googlePlace;
};
// This helper is for converting an Amazon Place object to a new Google Place class
const convertAmazonPlaceToGoogleNewPlace = (amazonPlaceObject, fields, googlePlace = null) => {
    const place = amazonPlaceObject.Place;
    // If a Google Place object wasn't passed in, then create a new one
    if (!googlePlace) {
        googlePlace = new MigrationPlace({
            id: amazonPlaceObject.PlaceId,
        });
    }
    // fields is required, so the only way to include all is by passing ['*'] or
    // specifying each individual field
    let includeAllFields = false;
    if (fields.includes("*")) {
        includeAllFields = true;
    }
    if (includeAllFields || fields.includes("displayName")) {
        googlePlace.displayName = place.Label.split(",")[0];
    }
    if (includeAllFields || fields.includes("formattedAddress")) {
        googlePlace.formattedAddress = place.Label;
    }
    if (includeAllFields || fields.includes("location")) {
        const point = place.Geometry.Point;
        googlePlace.location = new MigrationLatLng(point[1], point[0]);
    }
    if (includeAllFields || fields.includes("utcOffsetMinutes")) {
        // Our time zone offset is given in seconds, but Google's uses minutes
        let timeZoneOffsetInMinutes;
        if (place.TimeZone) {
            timeZoneOffsetInMinutes = place.TimeZone.Offset / 60;
            googlePlace.utcOffsetMinutes = timeZoneOffsetInMinutes;
        }
    }
    return googlePlace;
};
class MigrationPlacesService {
    findPlaceFromQuery(request, callback) {
        const query = request.query;
        const fields = request.fields;
        const locationBias = request.locationBias; // optional
        const input = {
            IndexName: this._placeIndexName,
            Text: query,
            MaxResults: 10, // findPlaceFromQuery usually returns a single result
        };
        if (locationBias) {
            const lngLat = LatLngToLngLat(locationBias);
            if (lngLat) {
                input.BiasPosition = lngLat;
            }
        }
        const command = new SearchPlaceIndexForTextCommand(input);
        this._client
            .send(command)
            .then((response) => {
            const googleResults = [];
            const results = response.Results;
            if (results.length !== 0) {
                results.forEach(function (place) {
                    const placeResponse = convertAmazonPlaceToGoogle(place, fields, false);
                    googleResults.push(placeResponse);
                });
            }
            callback(googleResults, PlacesServiceStatus.OK);
        })
            .catch((error) => {
            console.error(error);
            callback([], PlacesServiceStatus.UNKNOWN_ERROR);
        });
    }
    getDetails(request, callback) {
        const placeId = request.placeId;
        const fields = request.fields; // optional
        const input = {
            IndexName: this._placeIndexName,
            PlaceId: placeId, // required
        };
        const command = new GetPlaceCommand(input);
        this._client
            .send(command)
            .then((response) => {
            const place = response.Place;
            const googlePlace = convertAmazonPlaceToGoogle({ Place: place, PlaceId: placeId }, fields, true);
            callback(googlePlace, PlacesServiceStatus.OK);
        })
            .catch((error) => {
            console.error(error);
            callback(null, PlacesServiceStatus.UNKNOWN_ERROR);
        });
    }
    textSearch(request, callback) {
        const query = request.query; // optional
        const locationBias = request.location; // optional
        const bounds = request.bounds; // optional
        const language = request.language; // optional
        const region = request.region; // optional
        const input = {
            IndexName: this._placeIndexName,
            Text: query, // required
        };
        // If bounds is specified, then location bias is ignored
        if (bounds) {
            const latLngBounds = new MigrationLatLngBounds(bounds);
            const southWest = latLngBounds.getSouthWest();
            const northEast = latLngBounds.getNorthEast();
            input.FilterBBox = [southWest.lng(), southWest.lat(), northEast.lng(), northEast.lat()];
        }
        else if (locationBias) {
            const lngLat = LatLngToLngLat(locationBias);
            if (lngLat) {
                input.BiasPosition = lngLat;
            }
        }
        if (language) {
            input.Language = language;
        }
        if (region) {
            input.FilterCountries = [region];
        }
        const command = new SearchPlaceIndexForTextCommand(input);
        this._client
            .send(command)
            .then((response) => {
            const googleResults = [];
            const results = response.Results;
            if (results.length !== 0) {
                results.forEach(function (place) {
                    // Include all supported fields as in findPlaceFromQuery,
                    // but not the additional fields for getDetails
                    const placeResponse = convertAmazonPlaceToGoogle(place, ["ALL"], false);
                    googleResults.push(placeResponse);
                });
            }
            callback(googleResults, PlacesServiceStatus.OK);
        })
            .catch((error) => {
            console.error(error);
            callback([], PlacesServiceStatus.UNKNOWN_ERROR);
        });
    }
}
class MigrationPlace {
    constructor(options) {
        this.id = options.id;
        if (options.requestedLanguage) {
            this.requestedLanguage = options.requestedLanguage;
        }
    }
    fetchFields(options) {
        const placeId = this.id;
        const requestedLanguage = this.requestedLanguage;
        const fields = options.fields; // required
        const input = {
            IndexName: MigrationPlace._placeIndexName,
            PlaceId: placeId, // required
        };
        if (requestedLanguage) {
            input.Language = requestedLanguage;
        }
        return new Promise((resolve, reject) => {
            const command = new GetPlaceCommand(input);
            MigrationPlace._client
                .send(command)
                .then((response) => {
                const place = response.Place;
                // Pass in this reference so it will get updated, but we also return it as well
                const newPlace = convertAmazonPlaceToGoogleNewPlace({ Place: place, PlaceId: placeId }, fields, this);
                resolve({
                    place: newPlace,
                });
            })
                .catch((error) => {
                console.error(error);
                reject({
                    status: PlacesServiceStatus.UNKNOWN_ERROR,
                });
            });
        });
    }
    toJSON() {
        const jsonObject = {};
        // Iterating over the properties on our instance like this will only give us
        // back non-null properties, so any fields that weren't requested will be omitted for us
        for (const property in this) {
            const value = this[property];
            // Handle special-case for location property that needs to return its own JSON object
            // Everything else that's a primitive (boolean/string/number) can just return the value as-is
            if (property === "location") {
                jsonObject[property] = value.toJSON();
            }
            else {
                jsonObject[property] = value;
            }
        }
        return jsonObject;
    }
    static searchByText(request) {
        const query = request.textQuery || request.query; // textQuery is the new preferred field, query is deprecated but still allowed
        const locationBias = request.locationBias; // optional
        const bounds = request.locationRestriction; // optional
        const language = request.language; // optional
        const fields = request.fields || ["*"]; // optional
        const maxResultCount = request.maxResultCount; // optional
        const input = {
            IndexName: this._placeIndexName,
            Text: query, // required
        };
        // If bounds is specified, then location bias is ignored
        if (bounds) {
            const latLngBounds = new MigrationLatLngBounds(bounds);
            const southWest = latLngBounds.getSouthWest();
            const northEast = latLngBounds.getNorthEast();
            input.FilterBBox = [southWest.lng(), southWest.lat(), northEast.lng(), northEast.lat()];
        }
        else if (locationBias) {
            const lngLat = LatLngToLngLat(locationBias);
            if (lngLat) {
                input.BiasPosition = lngLat;
            }
        }
        if (language) {
            input.Language = language;
        }
        if (maxResultCount) {
            input.MaxResults = maxResultCount;
        }
        return new Promise((resolve, reject) => {
            const command = new SearchPlaceIndexForTextCommand(input);
            MigrationPlace._client
                .send(command)
                .then((response) => {
                const googlePlaces = [];
                const results = response.Results;
                if (results.length !== 0) {
                    results.forEach(function (place) {
                        const newPlace = convertAmazonPlaceToGoogleNewPlace(place, fields);
                        googlePlaces.push(newPlace);
                    });
                }
                resolve({
                    places: googlePlaces,
                });
            })
                .catch((error) => {
                console.error(error);
                reject({
                    status: PlacesServiceStatus.UNKNOWN_ERROR,
                });
            });
        });
    }
}
class MigrationAutocompleteService {
    getQueryPredictions(request, callback) {
        const query = request.input;
        const location = request.location; // optional
        const locationBias = request.locationBias; // optional
        const bounds = request.bounds || request.locationRestriction; // optional
        const language = request.language; // optional
        const input = {
            IndexName: this._placeIndexName,
            Text: query, // required
        };
        // Handle location/bounds restrictions. bounds and location have been deprecated, and in some cases
        // have actually been removed (although still mentioned in the documentation as only deprecated).
        //   * locationBias is the top preferred field, and can be MigrationLatLng|LatLngLiteral|MigrationLatLngBounds|LatLngBoundsLiteral
        //   * bounds / locationRestriction is the next preferred field
        //   * location is the final field that is checked
        let inputBounds, inputLocation;
        if (locationBias) {
            // MigrationLatLng|LatLngLiteral
            if (locationBias.lat !== undefined && locationBias.lng !== undefined) {
                inputLocation = new MigrationLatLng(locationBias);
            } /* MigrationLatLngBounds|LatLngBoundsLiteral */
            else {
                inputBounds = new MigrationLatLngBounds(locationBias);
            }
        }
        else if (bounds) {
            inputBounds = new MigrationLatLngBounds(bounds);
        }
        else if (location) {
            inputLocation = new MigrationLatLng(location);
        }
        // If bounds was found, then location is ignored
        if (inputBounds) {
            const southWest = inputBounds.getSouthWest();
            const northEast = inputBounds.getNorthEast();
            input.FilterBBox = [southWest.lng(), southWest.lat(), northEast.lng(), northEast.lat()];
        }
        else if (inputLocation) {
            const lngLat = LatLngToLngLat(inputLocation);
            if (lngLat) {
                input.BiasPosition = lngLat;
            }
        }
        if (language) {
            input.Language = language;
        }
        const command = new SearchPlaceIndexForSuggestionsCommand(input);
        this._client
            .send(command)
            .then((response) => {
            const googlePredictions = [];
            const results = response.Results;
            if (results && results.length !== 0) {
                results.forEach(function (result) {
                    const prediction = {
                        description: result.Text,
                    };
                    if (result.PlaceId) {
                        prediction.place_id = result.PlaceId;
                    }
                    googlePredictions.push(prediction);
                });
            }
            callback(googlePredictions, PlacesServiceStatus.OK);
        })
            .catch((error) => {
            console.error(error);
            callback([], PlacesServiceStatus.UNKNOWN_ERROR);
        });
    }
    // getPlacePredictions has a similar behavior as getQueryPredictions, except it omits query predictions,
    // so it only returns predictions that have a place_id
    getPlacePredictions(request, callback) {
        return new Promise((resolve) => {
            this.getQueryPredictions(request, (predictions, status) => {
                // Filter out predictions that don't have a place_id
                const filteredPredictions = predictions.filter((prediction) => {
                    return prediction.place_id;
                });
                // If a callback was given, invoke it before resolving the promise
                if (callback) {
                    callback(filteredPredictions, status);
                }
                resolve({
                    predictions: filteredPredictions,
                });
            });
        });
    }
}
class MigrationAutocomplete {
    constructor(inputField, opts) {
        _MigrationAutocomplete_maplibreGeocoder.set(this, void 0);
        _MigrationAutocomplete_bounds.set(this, void 0);
        _MigrationAutocomplete_strictBounds.set(this, false);
        _MigrationAutocomplete_fields.set(this, void 0);
        _MigrationAutocomplete_place.set(this, void 0);
        // Same base geocoder options as SearchBox, except we add omitSuggestionsWithoutPlaceId
        // so that we only get results with a PlaceId
        const maplibreGeocoderOptions = {
            enableAll: true,
            omitSuggestionsWithoutPlaceId: true,
        };
        if (inputField.placeholder) {
            maplibreGeocoderOptions.placeholder = inputField.placeholder;
        }
        __classPrivateFieldSet(this, _MigrationAutocomplete_maplibreGeocoder, buildAmazonLocationMaplibreGeocoder(this._client, this._placeIndexName, maplibreGeocoderOptions), "f");
        const geocoder = __classPrivateFieldGet(this, _MigrationAutocomplete_maplibreGeocoder, "f").getPlacesGeocoder();
        geocoder.addTo(inputField.parentElement);
        if (inputField.className) {
            geocoder.container.className = `${inputField.className} ${geocoder.container.className}`;
        }
        if (inputField.id) {
            geocoder._inputEl.id = inputField.id;
        }
        inputField.remove();
        if (opts) {
            this.setOptions(opts);
        }
    }
    getBounds() {
        return __classPrivateFieldGet(this, _MigrationAutocomplete_bounds, "f");
    }
    getFields() {
        return __classPrivateFieldGet(this, _MigrationAutocomplete_fields, "f");
    }
    getPlace() {
        return __classPrivateFieldGet(this, _MigrationAutocomplete_place, "f");
    }
    setBounds(bounds) {
        __classPrivateFieldSet(this, _MigrationAutocomplete_bounds, new MigrationLatLngBounds(bounds), "f");
        // Google's setBounds is used to bias, but the geocoder's bounds is a firm restriction, so
        // if strictBounds isn't specified, then we use the center of the input bounds to bias
        if (__classPrivateFieldGet(this, _MigrationAutocomplete_strictBounds, "f")) {
            const southWest = __classPrivateFieldGet(this, _MigrationAutocomplete_bounds, "f").getSouthWest();
            const northEast = __classPrivateFieldGet(this, _MigrationAutocomplete_bounds, "f").getNorthEast();
            const boundingBox = {
                longitudeSW: southWest.lng(),
                latitudeSW: southWest.lat(),
                longitudeNE: northEast.lng(),
                latitudeNE: northEast.lat(),
            };
            __classPrivateFieldGet(this, _MigrationAutocomplete_maplibreGeocoder, "f").setBoundingBox(boundingBox);
        }
        else {
            const center = __classPrivateFieldGet(this, _MigrationAutocomplete_bounds, "f").getCenter();
            __classPrivateFieldGet(this, _MigrationAutocomplete_maplibreGeocoder, "f").setBiasPosition({
                latitude: center.lat(),
                longitude: center.lng(),
            });
        }
    }
    setFields(fields) {
        __classPrivateFieldSet(this, _MigrationAutocomplete_fields, fields, "f");
    }
    setOptions(options) {
        // Read in strictBounds option first since it will determine how
        // the bounds option is consumed
        if (typeof (options === null || options === void 0 ? void 0 : options.strictBounds) === "boolean") {
            __classPrivateFieldSet(this, _MigrationAutocomplete_strictBounds, options.strictBounds, "f");
        }
        if (options === null || options === void 0 ? void 0 : options.bounds) {
            this.setBounds(options.bounds);
        }
        if (options === null || options === void 0 ? void 0 : options.fields) {
            __classPrivateFieldSet(this, _MigrationAutocomplete_fields, options.fields, "f");
        }
    }
    addListener(eventName, handler, listenerType = "on") {
        if (eventName == "place_changed") {
            // This event is triggered if the user selects either a place from the retrieved suggestions
            const resultsWrappedHandler = (results) => {
                if (results.place) {
                    // The fields could be set later, so we need to query again before converting the place
                    const fields = __classPrivateFieldGet(this, _MigrationAutocomplete_fields, "f") || ["ALL"];
                    __classPrivateFieldSet(this, _MigrationAutocomplete_place, convertAmazonPlaceToGoogle(results.place.properties, fields, true), "f");
                    // When the user picks a prediction, the geocoder displays the updated results
                    // by default (e.g. drops down the single chosen prediction).
                    // Google's widget does not do this, so in order to force the
                    // results to collapse, we need to focus and then unfocus the input element.
                    const inputElement = __classPrivateFieldGet(this, _MigrationAutocomplete_maplibreGeocoder, "f").getPlacesGeocoder()._inputEl;
                    inputElement.focus();
                    inputElement.blur();
                    handler();
                    if (listenerType == "once") {
                        __classPrivateFieldGet(this, _MigrationAutocomplete_maplibreGeocoder, "f").getPlacesGeocoder().off("results", resultsWrappedHandler);
                        __classPrivateFieldGet(this, _MigrationAutocomplete_maplibreGeocoder, "f").getPlacesGeocoder().off("result", resultWrappedHandler);
                    }
                }
            };
            __classPrivateFieldGet(this, _MigrationAutocomplete_maplibreGeocoder, "f").getPlacesGeocoder().on("results", resultsWrappedHandler);
            // This event is triggered if the user re-selects the single place that had been previously selected
            // from the list of suggestions
            const resultWrappedHandler = (result) => {
                // The fields could be set later, so we need to query again before converting the place
                const fields = __classPrivateFieldGet(this, _MigrationAutocomplete_fields, "f") || ["ALL"];
                __classPrivateFieldSet(this, _MigrationAutocomplete_place, convertAmazonPlaceToGoogle(result.result.properties, fields, true), "f");
                handler();
                if (listenerType == "once") {
                    __classPrivateFieldGet(this, _MigrationAutocomplete_maplibreGeocoder, "f").getPlacesGeocoder().off("result", resultWrappedHandler);
                    __classPrivateFieldGet(this, _MigrationAutocomplete_maplibreGeocoder, "f").getPlacesGeocoder().off("results", resultsWrappedHandler);
                }
            };
            __classPrivateFieldGet(this, _MigrationAutocomplete_maplibreGeocoder, "f").getPlacesGeocoder().on("result", resultWrappedHandler);
            return {
                instance: this,
                eventName: eventName,
                resultHandler: resultWrappedHandler,
                resultsHandler: resultsWrappedHandler,
            };
        }
    }
    _getMaplibreGeocoder() {
        return __classPrivateFieldGet(this, _MigrationAutocomplete_maplibreGeocoder, "f");
    }
    _setMapLibreGeocoder(geocoder) {
        __classPrivateFieldSet(this, _MigrationAutocomplete_maplibreGeocoder, geocoder, "f");
    }
}
_MigrationAutocomplete_maplibreGeocoder = new WeakMap(), _MigrationAutocomplete_bounds = new WeakMap(), _MigrationAutocomplete_strictBounds = new WeakMap(), _MigrationAutocomplete_fields = new WeakMap(), _MigrationAutocomplete_place = new WeakMap();
class MigrationSearchBox {
    constructor(inputField, opts) {
        _MigrationSearchBox_maplibreGeocoder.set(this, void 0);
        _MigrationSearchBox_bounds.set(this, void 0);
        _MigrationSearchBox_places.set(this, void 0);
        const maplibreGeocoderOptions = {
            enableAll: true,
        };
        if (inputField.placeholder) {
            maplibreGeocoderOptions.placeholder = inputField.placeholder;
        }
        __classPrivateFieldSet(this, _MigrationSearchBox_maplibreGeocoder, buildAmazonLocationMaplibreGeocoder(this._client, this._placeIndexName, maplibreGeocoderOptions), "f");
        if (opts === null || opts === void 0 ? void 0 : opts.bounds) {
            this.setBounds(opts.bounds);
        }
        const geocoder = __classPrivateFieldGet(this, _MigrationSearchBox_maplibreGeocoder, "f").getPlacesGeocoder();
        geocoder.addTo(inputField.parentElement);
        if (inputField.className) {
            geocoder.container.className = `${inputField.className} ${geocoder.container.className}`;
        }
        if (inputField.id) {
            geocoder._inputEl.id = inputField.id;
        }
        inputField.remove();
    }
    getBounds() {
        return __classPrivateFieldGet(this, _MigrationSearchBox_bounds, "f");
    }
    setBounds(bounds) {
        __classPrivateFieldSet(this, _MigrationSearchBox_bounds, new MigrationLatLngBounds(bounds), "f");
        // TODO: Google's setBounds is used to bias, but the geocoder's bounds is a firm restriction, so
        // for now we use the center of the input bounds to bias
        const center = __classPrivateFieldGet(this, _MigrationSearchBox_bounds, "f").getCenter();
        __classPrivateFieldGet(this, _MigrationSearchBox_maplibreGeocoder, "f").setBiasPosition({
            latitude: center.lat(),
            longitude: center.lng(),
        });
    }
    getPlaces() {
        return __classPrivateFieldGet(this, _MigrationSearchBox_places, "f");
    }
    addListener(eventName, handler, listenerType = "on") {
        if (eventName == "places_changed") {
            // This event is triggered if the user selects either a place or query suggestion
            // from the retrieved suggestions
            const resultsWrappedHandler = (results) => {
                var _a;
                if (results.place || ((_a = results.features) === null || _a === void 0 ? void 0 : _a.length)) {
                    if (results.place) {
                        __classPrivateFieldSet(this, _MigrationSearchBox_places, [convertAmazonPlaceToGoogle(results.place.properties, ["ALL"], true)], "f");
                    }
                    else {
                        __classPrivateFieldSet(this, _MigrationSearchBox_places, results.features.map((result) => {
                            return convertAmazonPlaceToGoogle(result.properties, ["ALL"], true);
                        }), "f");
                    }
                    // When the user picks a prediction, the geocoder displays the updated results
                    // by default (e.g. drops down the single chosen prediction, or a list of the results
                    // for the query string). Google's widget does not do this, so in order to force the
                    // results to collapse, we need to focus and then unfocus the input element.
                    const inputElement = __classPrivateFieldGet(this, _MigrationSearchBox_maplibreGeocoder, "f").getPlacesGeocoder()._inputEl;
                    inputElement.focus();
                    inputElement.blur();
                    handler();
                    if (listenerType == "once") {
                        __classPrivateFieldGet(this, _MigrationSearchBox_maplibreGeocoder, "f").getPlacesGeocoder().off("results", resultsWrappedHandler);
                        __classPrivateFieldGet(this, _MigrationSearchBox_maplibreGeocoder, "f").getPlacesGeocoder().off("result", resultWrappedHandler);
                    }
                }
            };
            __classPrivateFieldGet(this, _MigrationSearchBox_maplibreGeocoder, "f").getPlacesGeocoder().on("results", resultsWrappedHandler);
            // This event is triggered if the user selects a place from a list of query suggestions
            const resultWrappedHandler = (result) => {
                __classPrivateFieldSet(this, _MigrationSearchBox_places, [convertAmazonPlaceToGoogle(result.result.properties, ["ALL"], true)], "f");
                handler();
                if (listenerType == "once") {
                    __classPrivateFieldGet(this, _MigrationSearchBox_maplibreGeocoder, "f").getPlacesGeocoder().off("result", resultWrappedHandler);
                    __classPrivateFieldGet(this, _MigrationSearchBox_maplibreGeocoder, "f").getPlacesGeocoder().off("results", resultsWrappedHandler);
                }
            };
            __classPrivateFieldGet(this, _MigrationSearchBox_maplibreGeocoder, "f").getPlacesGeocoder().on("result", resultWrappedHandler);
            return {
                instance: this,
                eventName: eventName,
                resultHandler: resultWrappedHandler,
                resultsHandler: resultsWrappedHandler,
            };
        }
    }
    _getMaplibreGeocoder() {
        return __classPrivateFieldGet(this, _MigrationSearchBox_maplibreGeocoder, "f");
    }
    _setMapLibreGeocoder(geocoder) {
        __classPrivateFieldSet(this, _MigrationSearchBox_maplibreGeocoder, geocoder, "f");
    }
}
_MigrationSearchBox_maplibreGeocoder = new WeakMap(), _MigrationSearchBox_bounds = new WeakMap(), _MigrationSearchBox_places = new WeakMap();
export { MigrationAutocomplete, MigrationAutocompleteService, MigrationPlace, MigrationPlacesService, MigrationSearchBox, convertAmazonPlaceToGoogle, };
