import { LocationClient } from "@aws-sdk/client-location";
import { AddListenerResponse, DirectionsStatus, LatLngLiteral, MigrationLatLng, MigrationLatLngBounds } from "./googleCommon";
import { MigrationMap } from "./maps";
import { MigrationMarker } from "./markers";
import { MigrationPlacesService } from "./places";
interface Distance {
    text: string;
    value: number;
}
interface Duration {
    text: string;
    value: number;
}
interface DrivingOptions {
    departureTime: Date;
}
interface DirectionsGeocodedWaypoint {
    partial_match?: boolean;
    place_id?: string;
    types?: string[];
}
interface Place {
    location?: MigrationLatLng | null | LatLngLiteral;
    placeId?: string;
    query?: string;
}
interface DirectionsStep {
    distance?: Distance;
    duration?: Duration;
    encoded_lat_lngs?: string;
    end_location: MigrationLatLng;
    instructions?: string;
    maneuver?: string;
    path?: MigrationLatLng[];
    start_location: MigrationLatLng;
    steps?: DirectionsStep[];
    travel_mode: TravelMode;
}
interface DirectionsLeg {
    distance?: Distance;
    duration?: Duration;
    end_address: string;
    end_location: MigrationLatLng;
    start_address: string;
    start_location: MigrationLatLng;
    steps: DirectionsStep[];
}
interface DirectionsRoute {
    bounds: MigrationLatLngBounds;
    legs: DirectionsLeg[];
}
export declare enum UnitSystem {
    IMPERIAL = 0,
    METRIC = 1
}
export declare enum TravelMode {
    DRIVING = "DRIVING",
    WALKING = "WALKING",
    BICYCLING = "BICYCLING",
    TRANSIT = "TRANSIT",
    TWO_WHEELER = "TWO_WHEELER"
}
interface DirectionsWaypoint {
    location?: string | MigrationLatLng | LatLngLiteral | Place;
    stopover?: boolean;
}
interface DirectionsRequest {
    avoidFerries?: boolean;
    avoidHighways?: boolean;
    avoidTolls?: boolean;
    destination: string | MigrationLatLng | Place | LatLngLiteral;
    drivingOptions?: DrivingOptions;
    language?: string | null;
    optimizeWaypoints?: boolean;
    origin: string | MigrationLatLng | Place | LatLngLiteral;
    provideRouteAlternatives?: boolean;
    region?: string | null;
    travelMode: TravelMode;
    unitSystem?: UnitSystem;
    waypoints?: DirectionsWaypoint[];
}
interface DirectionsResult {
    geocoded_waypoints?: DirectionsGeocodedWaypoint[];
    request: DirectionsRequest;
    routes: DirectionsRoute[];
    status: DirectionsStatus;
}
export declare enum DistanceMatrixStatus {
    INVALID_REQUEST = "INVALID_REQUEST",
    MAX_DIMENSIONS_EXCEEDED = "MAX_DIMENSIONS_EXCEEDED",
    MAX_ELEMENTS_EXCEEDED = "MAX_ELEMENTS_EXCEEDED",
    OK = "OK",
    OVER_QUERY_LIMIT = "OVER_QUERY_LIMIT",
    REQUEST_DENIED = "REQUEST_DENIED",
    UNKNOWN_ERROR = "UNKNOWN_ERROR"
}
export declare enum DistanceMatrixElementStatus {
    OK = "OK",
    ZERO_RESULTS = "ZERO_RESULTS",
    NOT_FOUND = "NOT_FOUND"
}
interface DistanceMatrixResponseElement {
    distance: Distance;
    duration: Duration;
    status: DistanceMatrixElementStatus;
}
interface DistanceMatrixResponseRow {
    elements: DistanceMatrixResponseElement[];
}
interface DistanceMatrixRequest {
    destinations: (string | MigrationLatLng | Place | LatLngLiteral)[];
    origins: (string | MigrationLatLng | Place | LatLngLiteral)[];
    travelMode: TravelMode;
    avoidFerries?: boolean;
    avoidHighways?: boolean;
    avoidTolls?: boolean;
    drivingOptions?: DrivingOptions;
    language?: string | null;
    region?: string | null;
    unitSystem?: UnitSystem;
}
interface DistanceMatrixResponse {
    destinationAddresses: string[];
    originAddresses: string[];
    rows: DistanceMatrixResponseRow[];
}
declare class MigrationDirectionsService {
    _client: LocationClient;
    _routeCalculatorName: string;
    _placesService: MigrationPlacesService;
    route(options: DirectionsRequest, callback?: any): Promise<DirectionsResult>;
    _convertAmazonResponseToGoogleResponse(response: any, options: any, originResponse: any, destinationResponse: any, waypointResponses?: any): DirectionsResult;
    _constructGeocodedWaypointsFromResponses(originResponse: any, destinationResponse: any, waypointResponses?: any): DirectionsGeocodedWaypoint[];
    _constructGeocodedWaypoint(locationResponse: any): Record<"place_id", unknown> | Record<"types", unknown>;
}
declare class MigrationDirectionsRenderer {
    #private;
    constructor(options?: any);
    addListener(eventName: any, handler: any, listenerType?: string): AddListenerResponse;
    getDirections(): any;
    getMap(): MigrationMap;
    setMap(map: any): void;
    setDirections(directions: any): void;
    setOptions(options?: any): void;
    _clearDirections(): void;
    _getMarkers(): MigrationMarker[];
    _getMarkerOptions(): any;
    _getPreserveViewport(): boolean;
    _getSuppressMarkers(): boolean;
    _getSuppressPolylines(): boolean;
    _getOnDirectionsChangedListeners(): any[];
    _getOnceDirectionsChangedListeners(): any[];
    _setOnDirectionsChangedListeners(listeners: any): void;
    _setOnceDirectionsChangedListeners(listeners: any): void;
}
declare class MigrationDistanceMatrixService {
    _client: LocationClient;
    _routeCalculatorName: string;
    _placesService: MigrationPlacesService;
    getDistanceMatrix(request: DistanceMatrixRequest, callback?: any): Promise<DistanceMatrixResponse>;
    _convertAmazonResponseToGoogleResponse(calculateRouteMatrixResponse: any, originsResponse: any, destinationsResponse: any, request: any): DistanceMatrixResponse;
}
export { MigrationDirectionsService, MigrationDirectionsRenderer, MigrationDistanceMatrixService };
