import { LocationClient } from "@aws-sdk/client-location";
import { AddListenerResponse, LatLngBoundsLike, LatLngBoundsLiteral, LatLngLike, LatLngLiteral, MigrationLatLng, MigrationLatLngBounds } from "./googleCommon";
interface AutocompletePrediction {
    description: string;
    place_id: string;
}
interface AutocompleteResponse {
    predictions: AutocompletePrediction[];
}
interface PlaceOptions {
    id: string;
    requestedLanguage?: string | null;
    requestedRegion?: string | null;
}
interface FindPlaceFromQueryRequest {
    fields: string[];
    language?: string | null;
    locationBias?: LatLngLike | LatLngBoundsLike | string;
    query: string;
}
interface FetchFieldsRequest {
    fields: string[];
}
interface SearchByTextRequest {
    fields: string[];
    includedType?: string;
    isOpenNow?: boolean;
    language?: string;
    locationBias?: MigrationLatLng | LatLngLiteral | MigrationLatLngBounds | LatLngBoundsLiteral;
    locationRestriction?: MigrationLatLngBounds | LatLngBoundsLiteral;
    maxResultCount?: number;
    minRating?: number;
    query?: string;
    region?: string;
    textQuery?: string;
    useStrictTypeFiltering?: boolean;
}
declare const convertAmazonPlaceToGoogle: (placeObject: any, fields: any, includeDetailFields: any) => {};
declare class MigrationPlacesService {
    _client: LocationClient;
    _placeIndexName: string;
    findPlaceFromQuery(request: FindPlaceFromQueryRequest, callback: any): void;
    getDetails(request: any, callback: any): void;
    textSearch(request: any, callback: any): void;
}
declare class MigrationPlace {
    static _client: LocationClient;
    static _placeIndexName: string;
    displayName?: string | null;
    formattedAddress?: string | null;
    id: string;
    location?: MigrationLatLng | null;
    requestedLanguage?: string | null;
    requestedRegion?: string | null;
    utcOffsetMinutes?: number | null;
    constructor(options: PlaceOptions);
    fetchFields(options: FetchFieldsRequest): Promise<{
        place: MigrationPlace;
    }>;
    toJSON(): {};
    static searchByText(request: SearchByTextRequest): Promise<{
        places: MigrationPlace[];
    }>;
}
declare class MigrationAutocompleteService {
    _client: LocationClient;
    _placeIndexName: string;
    getQueryPredictions(request: any, callback: any): void;
    getPlacePredictions(request: any, callback?: any): Promise<AutocompleteResponse>;
}
declare class MigrationAutocomplete {
    #private;
    _client: LocationClient;
    _placeIndexName: string;
    constructor(inputField: HTMLInputElement, opts?: any);
    getBounds(): MigrationLatLngBounds;
    getFields(): any;
    getPlace(): any;
    setBounds(bounds: any): void;
    setFields(fields: any): void;
    setOptions(options: any): void;
    addListener(eventName: any, handler: any, listenerType?: string): AddListenerResponse;
    _getMaplibreGeocoder(): any;
    _setMapLibreGeocoder(geocoder: any): void;
}
declare class MigrationSearchBox {
    #private;
    _client: LocationClient;
    _placeIndexName: string;
    constructor(inputField: HTMLInputElement, opts?: any);
    getBounds(): MigrationLatLngBounds;
    setBounds(bounds: any): void;
    getPlaces(): any;
    addListener(eventName: any, handler: any, listenerType?: string): AddListenerResponse;
    _getMaplibreGeocoder(): any;
    _setMapLibreGeocoder(geocoder: any): void;
}
export { MigrationAutocomplete, MigrationAutocompleteService, MigrationPlace, MigrationPlacesService, MigrationSearchBox, convertAmazonPlaceToGoogle, FindPlaceFromQueryRequest, };
